#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import os
import math
import koprhand
import koprfunc
import datavar

    # filter stars is field
def FilterRADE(ra, de, fov, stars):
    ralimit, delimit = [ ra - fov/2., ra + fov/2. ], [ de - fov/2., de + fov/2. ]
    stars0, stars1 = [x for x in stars if ralimit[0] <= float(x[1]) <= ralimit[1] and delimit[0] <= float(x[2]) <= delimit[1] ], []

    # if ra is out 0.- 360 limit    
    if ralimit[0] < 0.:
      ralimit[0] += 360.
      stars1 = [x for x in stars if ralimit[0] <= float(x[1]) <= 360. and delimit[0] <= float(x[2]) <= delimit[1] ]
    elif ralimit[1] > 360.:
      ralimit[1] -= 360.
      stars1 = [x for x in stars if 0. <= float(x[1]) <= ralimit[1] and delimit[0] <= float(x[2]) <= delimit[1] ]

    return stars0 + stars1

def Afiles(dec, fov):
    # dec limits
    declim = [ dec + fov/2., dec - fov/2. ]
    # dec normalisation
    for i in [0, 1]:
      if declim[i] < -90.:
        declim[i] = -90.
      elif declim[i] > 90.:
        declim[i] = 89.9999999999
      # obtaining file name range    
      declim[i] = int(math.floor(declim[i]/5.))

    declim.sort()
    apasskeys = []
    for j in range(declim[0],declim[1]+1):
      apasskeys.append(j)
      
    files = []
    for k in apasskeys:
      if k < 0:
        prefix = 'zm'
      else: 
        prefix = 'zp'
      files.append( (prefix + str(k*5) + '.txt').replace('-','') )

    return files  

def calcImag(starv, starg, stari):
    if (starg-stari) <= 2.1:
      imag = starv - (0.675)*(starg-stari) - (0.364) 
    else:
      imag = starv - (1.11)*(starg-stari) + (0.52)
    return imag

# Replacing koprfunc.VizierStars for offline
def GetLocalApass(header, maglim, magsrc, directory):
    magsrc = datavar.MagSources()[magsrc][0]
    params = koprhand.SettingsRead()

    xdim, ydim, ra, de = header['CRPIX1']*2., header['CRPIX2']*2., header['CRVAL1'], header['CRVAL2'] 

    try: 
      fov = header['CDELT1']*xdim
    except KeyError:
      cdelt1, cdelt2 = PixToWorld(header, 0, 0), PixToWorld(header, xdim, ydim)
      fov = sqrt((cdelt1[0]-cdelt2[0])*(cdelt1[0]-cdelt2[0]) + (cdelt1[1]-cdelt2[1])*(cdelt1[1]-cdelt2[1]))

    if abs(fov) > float(params[1][8]) and params[1][8] != '0': # Max FOV applied
      fov = float(params[1][8])
    
    sourcefiles = Afiles(de, fov)

    rawdata = []
    for item in sourcefiles:
      data = open( directory + item, 'r')
      for line in data.readlines():
        rawdata.append(line.split())
    
    # select stars in FOV
    fov = abs(fov)
    rawdata = FilterRADE(ra, de, fov, rawdata)

    # get star data
    starlist, blustar, yelstar, redstar, nonestar = [], [], [], [], []
    for star in rawdata:
      # get starra, starde, starbv, starmag
      # source format
      #        0         1          2       3      4      5      6     7
      #        dest     ra         de       B      V      g      r     i
      if magsrc == 'APASS B mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) ,float(star[3])
      elif magsrc == 'APASS V mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) ,float(star[4])
      elif magsrc == 'APASS calc. R mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) , (-0.153)*( float(star[6]) - float(star[7]) ) - ( 0.117 ) + float(star[6])
      elif magsrc == 'APASS calc. I mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) , calcImag( float(star[4]), float(star[5]), float(star[7]) )
      elif magsrc == 'APASS g\' mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) ,float(star[5])
      elif magsrc == 'APASS r\' mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) ,float(star[6])
      elif magsrc == 'APASS i\' mag':
        starra, starde, starbv, starmag = float(star[1]), float(star[2]), float(star[3]) - float(star[4]) ,float(star[7])

      x, y = koprfunc.WorldtoPix( header, starra, starde )
      # build star lists if star is in image
      if ( 0 < x < xdim ) and ( 0 < y < ydim ) and ( starmag < maglim ) : 
        starlist.append( [x, y, starmag, starbv] )
        if starbv < 0.5: 
          blustar.append( [x, y] )
        elif 0.5 <= starbv <= 0.8: 
          yelstar.append( [x, y] )
        elif starbv > 0.8: 
          redstar.append( [x, y] )

    return starlist, blustar, yelstar, redstar, nonestar    

def GetRawdata(ra, de, fov, directory):
    rawdata = []
    sourcefiles = Afiles(de, fov)
    for item in sourcefiles:
      data = open( directory + item, 'r')
      for line in data.readlines():
        rawdata.append(line.split())
    
    return FilterRADE(ra, de, fov, rawdata)

# append to koprfunc.readTYC(ra, de, fov, mapparams):
def getofflineTYC(ra, de, fov):
    rawdata = GetRawdata(ra, de, fov, './koprtyc2/')
    output = []
    # fake vizier output
    for line in rawdata:
      output.append(' '.join( [line[0], line[1], line[2], line[3], line[4], line[0].split('-')[0], line[0].split('-')[1], line[0].split('-')[2] ] ))

    return output

def GetLocalApassVIZ(ra, de, fov, directory):
    rawdata = GetRawdata(ra, de, fov, directory)
    output = []
    # fake vizier output
    for line in rawdata:
      output.append(' '.join( [line[0], line[1], line[2], line[3], str(0.850 * (float(line[4])-float(line[3]))) ] ))

    return output

def getofflineYBSC(ra, de, fov):
    data = open('./koprybsc5/bsc5.txt', 'r')
    output = []
    rawdata = data.readlines()
    data.close()

    for line in rawdata:
      line = line.split()
      output.append(' '.join( [line[0], line[1], line[2], line[3], line[4], line[5], line[6], line[7], line[8], line[0]]  ))

    return output
