#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import koprhand
import koprfunc
import koprdial
import koprclass
import koprfig
import comephem
import datavar
import datetime
from PyQt5 import QtCore, QtGui, QtWidgets
#import matplotlib
#matplotlib.use('Qt4Agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar

font = datavar.DefaultFont()
fontsmall = datavar.SmallFont()

## *******************************************
############## WIDGET ################
## *******************************************

class Visual(QtWidgets.QWidget):
    
    def __init__(self, parent=None):
        super(Visual, self).__init__()
        self.initUI()
        
    def initUI(self):               
        self.destig, self.refstars, self.mapparam, self.vizdata = '', [], [], []
        self.uidate, self.uitime = '', ''
        
        # Input grid
        grid = QtWidgets.QGridLayout()
        
        self.btnnew = QtWidgets.QPushButton('New observation', self) 
        self.btnnew.setFont(font)
        self.btnnew.clicked.connect(self.button_click_clear)
        
        labels = [ 'Comet name', 'Date YYYY/MM/DD', 'Time HH:MM (UT)', 'Estimate method', 'Coma diameter (\')', 'Degree of condensation', 'Tail length (\u00b0)', 'Position angle(\u00b0)' ]
        self.elements = []
        
        for i, name in enumerate(labels):
            label = QtWidgets.QLabel(name)
            label.setFont(font)
            grid.addWidget(label, i, 0 )
            if i in [3,5]:
              self.elements.append( QtWidgets.QComboBox(self) )
              self.elements[i].setStyleSheet("selection-color: black")
            elif i == 0: 
              self.elements.append( QtWidgets.QLineEdit(self) )
              self.elements[i].setDisabled(1)
            else:
              self.elements.append( QtWidgets.QLineEdit(self) )
            self.elements[i].setFont(font)
            self.elements[i].setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            grid.addWidget(self.elements[i], i, 1 )
        
        self.elements[1].setText(datetime.datetime.now().strftime("%Y/%m/%d"))
 
        self.measure = QtWidgets.QComboBox(self)
        for item in ['Measure Coma','Measure Tail','Measure Stars']:
          self.measure.addItem(item)
        self.obstype = QtWidgets.QComboBox(self)
        for item in ['Positive','Uncertain','Negative']:
          self.obstype.addItem(item)
        self.cbext = QtWidgets.QComboBox(self)
        for item in ['No Extinction','Winter','Spring/Autumn','Summer']:
          self.cbext.addItem(item)

        bmap = QtWidgets.QPushButton('Map parameters', self)
        bmap.clicked.connect(self.button_click_mapp)
        bedit = QtWidgets.QPushButton('Reference stars', self)
        bedit.clicked.connect(self.button_click_edit)
        bproc = QtWidgets.QPushButton('Process obs.', self)
        bproc.clicked.connect(self.button_click_calc)

        items = [ bmap, bedit, bproc ]
        for i, item in enumerate(items):
          item.setFont(font)
          grid.addWidget(item, i+8, 0 )

        items = [ self.measure, self.obstype, self.cbext]
        for i, item in enumerate(items):
          item.setFont(font)
          item.setStyleSheet("selection-color: black")
          grid.addWidget(item, i+8, 1 )

        self.comment = QtWidgets.QTextEdit(self)
        self.comment.setText('Comment')
        self.comment.setFont(font) 
        self.comment.setMaximumHeight(bproc.sizeHint().height() * 3)
        grid.addWidget(self.comment, 11, 0, 2, 2 )

        datavar.clickable(self.elements[1]).connect(self.datetime)
        datavar.clickable(self.elements[2]).connect(self.datetime)
        
        # Settings grid values for combobox
        for method in datavar.Methods():  
            self.elements[3].addItem(method)
          
        self.elements[5].addItem('No DC')
        for i in range(9):
            self.elements[5].addItem(str(i)+' ')
            self.elements[5].addItem(str(i)+'/')
        self.elements[5].addItem('9 ')

        # Info Grid
        self.infolabels = [ 'Dist.(AU)= ', 'r(AU)= ', 'Alt.(\u00b0)= ', 'Sun alt.(\u00b0)= ', 'Moon alt.(\u00b0)= ' ]
        self.info = []

        infogrid = QtWidgets.QGridLayout()
        for i, name in enumerate(self.infolabels):
            self.info.append(QtWidgets.QLabel(name))
            self.info[i].setFont(fontsmall)
            self.info[i].setMaximumHeight(bproc.sizeHint().height())
            self.info[i].setMinimumWidth(bproc.sizeHint().width())
            infogrid.addWidget(self.info[i], 0, i )

        # Map field
        self.sc = koprfig.VisMapDialog(self)
        self.shift_is_held = False
        self.refstar, self.coma_lst, self.tail_lst = {}, [], [ (1024,768) ]
      
        self.cid = self.sc.canvas.mpl_connect('button_press_event',self.onpickStar)
        self.cidsp = self.sc.canvas.mpl_connect('key_press_event', self.on_key_press)
        self.cidsr = self.sc.canvas.mpl_connect('key_release_event', self.on_key_release)

        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)

        vboxl = QtWidgets.QVBoxLayout() 
        vboxl.addWidget(self.btnnew)
        vboxl.addLayout(grid)
        vboxl.addStretch(1)

        vboxr = QtWidgets.QVBoxLayout() 
        vboxr.addWidget(self.mpl_toolbar)
        vboxr.addWidget(self.sc)
        vboxr.addLayout(infogrid)
        
        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr)
        
        ### test values
        #self.elements[0].setText('29P')
        #self.elements[1].setText('2017/10/01')
        #self.elements[2].setText('23:00')

        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - Visual observation')
        self.show()        

    def aftercoma(self):
      self.a = VisMeasure('comet')
      self.a.element[0].clicked.connect(self.button_measstar)
      self.a.element[1].clicked.connect(self.button_meastail)
      self.a.element[2].clicked.connect(self.button_meascoma)
      self.a.exec_()
      # measure tail ? measure stars ? open stars image? measure comet again?

    def aftertail(self):
      self.a = VisMeasure('tail')
      self.a.element[0].clicked.connect(self.button_measstar)
      self.a.element[1].clicked.connect(self.button_meastail)
      self.a.exec_()
      # measure stars ? open stars image? measure tail again?

    def button_measstar(self):
      self.measure.setCurrentIndex(2)
      self.a.close()

    def button_meastail(self):
      self.measure.setCurrentIndex(1)
      self.a.close()

    def button_meascoma(self):
      self.measure.setCurrentIndex(0)
      self.a.close()

    ## Key Pick events
    def on_key_press(self, event):
       if event.key == 'shift':
           self.shift_is_held = True
    
    def on_key_release(self, event):
       if event.key == 'shift':
           self.shift_is_held = False

    def comaredraw(self, xdata, ydata):
       self.coma_lst.append( (xdata, ydata) )
       x, y = list(zip(*self.coma_lst))
       self.sc.pt_plot.set_xdata(x)
       self.sc.pt_plot.set_ydata(y)
       self.sc.canvas.draw_idle()
       
    def tailredraw(self, xdata, ydata):
       self.tail_lst.append( (xdata, ydata) )
       x, y = list(zip(*self.tail_lst))
       self.sc.tpt_plot.set_xdata(x)
       self.sc.tpt_plot.set_ydata(y)
       self.sc.canvas.draw_idle()
       
    def onpickStar(self, event): 
      params = koprhand.SettingsRead()
      if event.xdata and event.ydata and event.button == 3:

        if self.measure.currentIndex() == 0:
            if len(self.coma_lst) == 0:
                self.comaredraw(event.xdata, event.ydata)
            elif len(self.coma_lst) == 1:
                dx = self.coma_lst[0][0] - event.xdata
                dy = self.coma_lst[0][1] - event.ydata
                r = (dx*dx + dy*dy)**(1./2.)/2.
                self.comacirc = plt.Circle((koprfunc.GetComaXY(self.header, self.mapparam)), r, color='r', fill=False)
                self.sc.pt_plot.set_xdata([])
                self.sc.pt_plot.set_ydata([])
                self.sc.ax.add_artist(self.comacirc) 
                self.comaredraw(event.xdata, event.ydata)
                coma = koprfunc.CalcTailpa(self.header, self.coma_lst[0][0], self.coma_lst[0][1], self.coma_lst[1][0], self.coma_lst[1][1])[0]
                self.elements[4].setText( str(round(coma*60,1)) )
                self.aftercoma()
            elif len(self.coma_lst) == 2:
                self.coma_lst = []
                self.comacirc.remove()
                self.comaredraw(event.xdata, event.ydata)
                
        elif self.measure.currentIndex() == 1:
            if len(self.tail_lst) == 1:  
                self.tailredraw(event.xdata, event.ydata)
            elif len(self.tail_lst) == 2:
                self.tail_lst.pop()
                self.tailredraw(event.xdata, event.ydata)
            cpx, cpy = koprfunc.GetComaXY(self.header, self.mapparam)
            tail, pa = koprfunc.CalcTailpa(self.header, cpx, cpy, event.xdata, event.ydata)
            self.elements[6].setText( str(round(tail,2)) )
            self.elements[7].setText( str(int(round(pa,0))) )
            self.aftertail()

        elif self.measure.currentIndex() == 2:
            distances, outlist = [], {} #print 'find closest star'
            for star in self.starlist[0]:
                if ((event.xdata-50) < star[0] < (event.xdata+50)) and ((event.ydata-50) < star[1] < (event.ydata+50)):
                    dx, dy = abs(star[0]-event.xdata), abs(star[1]-event.ydata)
                    c = dx*dx + dy*dy
                    distances.append(c) 
                    outlist[c] = star             
            try:
              x, y = outlist[min(distances)][0], outlist[min(distances)][1]
                         
              if event.button == 3 and self.shift_is_held:
                self.sc.starcirc[(x,y)].remove()
                self.sc.canvas.draw_idle()
                for star in self.refstars:
                  if star[1] == self.refstar[(x,y)][1] and star[2] == self.refstar[(x,y)][2]:
                    self.refstars.remove(star)
                    break    
                del self.refstar[(x,y)]
                del self.sc.starcirc[(x,y)]
            
              elif event.button == 3: #insert a AS degree
                text, ok = QtWidgets.QInputDialog.getText(self, 'Input star', 'Argelander degree:')                
                if text: 
                    world = koprfunc.PixToWorld(self.header, x,y)
                    destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())
                    staralt = round(comephem.StarAlt( world[0]/15., world[1], date, time, float(params[2][1]), float(params[2][2])),2)
                    self.refstar[(x,y)] = ['o', float(text), outlist[min(distances)][2], outlist[min(distances)][4], staralt, outlist[min(distances)][3], '' ]
                    self.refstars.append(self.refstar[(x,y)]) 
                    self.sc.starcirc[(x,y)] = plt.Circle((x, y), 20, color='r', fill = False)
                    self.sc.ax.add_artist(self.sc.starcirc[(x,y)])#print 'star'
                    self.sc.canvas.draw_idle() 

            except ValueError:
              koprdial.InfoDialog(1, 'No star found', 'No star found or no APASS data available for this star')  

    ## Map changes
    def MapPlot(self):
      #clean up map
      for key in list(self.refstar.keys()):
        self.sc.starcirc[key].remove()
        del self.sc.starcirc[key]
     
      self.refstar, self.refstars = {}, []
        
      try:
        self.comacirc.remove()
      except:
        pass
          
      self.sc.pt_plot.set_xdata([])
      self.sc.pt_plot.set_ydata([])

      self.sc.tpt_plot.set_xdata([])
      self.sc.tpt_plot.set_ydata([])

      #init new map
      self.header, self.starlist, self.vizdata = koprfunc.GenerateMap(self.mapparam, self.vizdata)
      destig = str(self.elements[0].text())
      # *** PLOT COMET POINTER ***
      cpx, cpy = koprfunc.GetComaXY(self.header, self.mapparam)
      self.sc.com_point.set_xdata( [cpx] )
      self.sc.com_point.set_ydata( [cpy] )
      # *** UPDATE TAIL Beggining ***
      self.tail_lst[0] = (cpx, cpy)

      # *** PLOT SCALE ***
      scale = float(self.mapparam[2])/2.2
      if scale > 1:
          scale, pxsize = str(int(scale))+' deg', (float(int(scale))*2048.)/float(self.mapparam[2]) 
      else:
          scale, pxsize = str(int(scale*6)*10 )+' arcmin', (float(int(scale*6))/6.*2048.) / float(self.mapparam[2])
      self.sc.scaletext.set_text(scale)

      self.sc.scalept.set_xdata( [1024*2-40, 1024*2-40-pxsize] )
      self.sc.scalept.set_ydata( [ 768*2-40,  768*2-40] )
      
      # *** PLOT LIMIT MAG ***
      self.sc.limmag.set_text('Limit mag.: '+self.mapparam[3])
      
      # *** PLOT STARS ***
      for i in range(len(self.sc.circle)):
        self.sc.circle[i].remove()
      self.sc.circle = []
      
      maglim = float(self.mapparam[3])
      if self.starlist[1]:
        for index, star in enumerate(self.starlist[1]):
          r = int (round( -1 * ( star[2] - maglim ) ) * 2 + 1)
          self.sc.circle.append(plt.Circle((star[0], star[1]), r, color='w'))
          self.sc.ax.add_artist(self.sc.circle[index])
      else:
        for index, star in enumerate(self.starlist[0]):
          r = int (round( -1 * ( star[2] - maglim ) ) * 2 + 1)
          self.sc.circle.append(plt.Circle((star[0], star[1]), r, color='w'))
          self.sc.ax.add_artist(self.sc.circle[index])
      
      self.sc.canvas.draw_idle()    

    def MapInit(self):
        params = koprhand.SettingsRead()
        destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text()) 
        comet, moonalt, moonp, sunalt = koprfunc.CometObsInf(destig, date, time, params[2][1], params[2][2], params[2][3])
        cometinfo = [ str(round(comet.dist,3)), str(round(comet.r,3)), str(round(comet.alt,2)), str(sunalt), str(moonalt) ]
        for i in range(len(cometinfo)):
          self.info[i].setText(self.infolabels[i]+cometinfo[i])
        
        ra, de = comephem.j2000rade( comet.ra, comet.de, date, time )
        lm = round((comet.mag+2.)*2. , 0 )/2.
        if lm < 6:
            lm = 6
        elif lm > 18:
            lm = 18
            
        if lm <= 7:
            cat = 'YBSC5'
        elif 7 < lm <= 12:
            cat = 'TYC-2'
        else:
            cat = 'UCAC4'

        if params[3][0] == 'Naked Eye':
            fov = 10.        
        else:
            if params[3][3]:
                fov = round(float(params[4][2]) / (float(params[3][3])/float(params[4][1])), 2)  
            elif params[3][4]: 
                fov = round(float(params[3][5]), 2) 
            
        # on change
        try:
          if self.mapparam[7] != destig or date + time != self.uidate+self.uitime:
            self.mapparam = [ str(round(ra*15.,2)), str(round(de,2)), str(fov), str(lm), cat, str(round(ra*15.,2)), str(round(de,2)), destig ]
            self.MapPlot()
        except IndexError:
            self.mapparam = [ str(round(ra*15.,2)), str(round(de,2)), str(fov), str(lm), cat, str(round(ra*15.,2)), str(round(de,2)), destig ]
            self.MapPlot()
        self.uidate, self.uitime = date, time

    def datetime(self):
        date, time = str(self.elements[1].text()), str(self.elements[2].text())
        if date and time:
          if len(date.split('/')) == 3 and len(time.split(':')) == 2:
            destig = str(self.elements[0].text())
            if len(date.split('/')[2]) == 2 and len(time.split(':')[1]) == 2 and destig != '':
                check = True
                for num in date.split('/'):
                  if not num.isdigit() or len(num) not in (4,2):
                    check = False
                    break
                for num in time.split(':'):
                  if not num.isdigit() or len(num) != 2:
                    check = False
                    break
                if check:
                  self.coma_lst = []
                  self.MapInit()
    
    def button_click_clear(self):
        params = koprhand.SettingsRead()
        text, ok = QtWidgets.QInputDialog.getText(self,  'New visual observation', 'Measure for location: '+params[2][0]+'\nTelescope: '+params[3][0]+'\nEyepiece: '+params[4][0]+'\n\nInput comet short name (e.g. 103P or C/2012 X1):')                
        if text: 
            eline = koprhand.loadcometid(text)
            if eline == '':
                koprdial.InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
            else:    
                self.elements[0].setText(text)
                for i in [2,4,6,7]:
                  self.elements[i].clear()
                self.elements[3].setCurrentIndex(0)
                self.elements[5].setCurrentIndex(0)
                self.measure.setCurrentIndex(0)
                self.obstype.setCurrentIndex(0)
                self.cbext.setCurrentIndex(0)
                self.comment.setText('Comment')
                self.destig, self.refstars, self.mapparam, self.vizdata = '', [], [], []
                self.header, self.starlist = [], [[],[]]
                self.btnnew.focusNextChild()

    def button_click_calc(self):
        params = koprhand.SettingsRead()
        code = koprclass.icqcode()
        locname, lon, lat, elev, gmt = params[2] 
        code.comet, code.date, code.time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())
        coma, tail, pa = str(self.elements[4].text()), str(self.elements[6].text()), str(self.elements[7].text())
        
        critical, errcount, errormsg, errordetmsg = koprhand.showdialogCal(self.refstars, coma, tail, pa)

        if critical == 1:
            koprdial.InfoDialog(1, 'Errors were encountered', str(errcount) + ' errors occured: ', errormsg, errordetmsg)
        else:        
            comet, moonalt, moonp, sunalt = koprfunc.CometObsInf(code.comet, code.date, code.time, lon, lat, elev)
            code.coma, code.otype, code.dc, code.method, code.name = float(coma), 'vis', str(self.elements[5].currentText()).replace('No DC','  '), str(self.elements[3].currentText())[0:1], comet.name 
            
            if self.obstype.currentText() == 'Negative':
                code.negsign, code.uncsign, specsign, obsmethod = '[', ' ', '!', 'O'
            elif self.obstype.currentText() == 'Uncertain':
                code.negsign, code.uncsign = ' ', ':'
            
            if self.mapparam[4] == 'TYC-2':
                code.magsrc = 'TK'
            elif self.mapparam[4] == 'UCAC4': 
                code.magsrc = 'AQ'
            elif self.mapparam[4] == 'YBSC5': 
                code.magsrc = 'YG'
    
            tname, code.ttype, code.apert = params[3][0], params[3][1], float(params[3][2])
            ename, efocl, efov = params[4]
            efocl, efov = float(efocl), float(efov)
            
            if params[3][1] == 'E':
              code.zoom, code.focl = 0, 0
            elif params[3][1] != 'E' and params[3][3]:
              tfocl = params[3][3] 
              code.zoom, code.focl = int( float(tfocl)/float(efocl)), float(tfocl)/float(code.apert)
            elif params[3][1] != 'E' and not params[3][3]:
              try:
                if params[3][4]:
                  code.zoom, code.focl = int(params[3][4]), 0
              except IndexError:
                pass
            
            code.obscode = params[0][0]
    
            # Nalezeni komety a kalkulace
            c, moonalt, moonp, sunalt = koprfunc.CometObsInf(code.comet, code.date, code.time, str(lon), str(lat), elev)
            comet, cometalt, rdist, ddist = c.name, round(c.alt,2), round(c.r,3), round(c.dist,3)
    
            if str(self.elements[6].text()):
                code.tail, code.tailpa = koprhand.degconvert(str(tail)), int(float(pa)) ##### Tail +pa input

            code.mag, code.extinct, xlist, ylist, slist, t, s, slope = koprfunc.ProcessObs(self.refstars, str(self.cbext.currentText()), cometalt, elev  )
            
            plot = [xlist, ylist, t, s, code.mag, code.comet+' '+code.date + ' ' + code.time+' '+code.obscode]
             
            if str(self.comment.toPlainText()).startswith('Comment'):
                code.obsnote = 'Comet Alt.:'+str(cometalt)+' AS:'+','.join(str(round(x,1)) for x in xlist)+' Star mags:'+','.join(str(y) for y in ylist) + ' ' + ','.join(str(n) for n in slist)+ '\n' #
            else: 
                code.obsnote = 'Comet Alt.:'+str(cometalt)+' AS:'+','.join(str(round(x,1)) for x in xlist)+' Star mags:'+','.join(str(y) for y in ylist) + ' ' + ','.join(str(n) for n in slist)+ '# ' + str(self.comment.toPlainText()).replace('\n','') +'\n' #
            
            output, output2 = code.build_icq()

            critical, warning, errcount, errormsg, errordetmsg = koprhand.showdialogOut( cometalt, code.mag, moonalt, moonp, sunalt, slope, 0, xlist ) #output errors
            if critical == 1:
                koprdial.InfoDialog(1, 'Errors were encountered', str(errcount) + ' errors occured: ', errormsg, errordetmsg)
            else:
                if warning == 1:
                    koprdial.InfoDialog(2, 'There are warnings', str(errcount) + ' warnings occured: ', errormsg, errordetmsg)
                a = koprdial.ObsOutput( output, output2 + ' ({:s} {:s}, {:s})'.format( params[0][1], params[0][2], locname ), code.obscode, plot )
                a.exec_()
        
    def button_click_edit(self):
        params = koprhand.SettingsRead()
        a = koprdial.StarDialog(refno = len(self.refstars), destig = str(self.elements[0].text()), date = str(self.elements[1].text()), time = str(self.elements[2].text()),\
                                 extinct = '' ,lon = params[2][1], lat = params[2][2], elev = params[2][3], stars = self.refstars )             
        a.exec_()
        if a.output == [] and self.refstars != []:
          for key in list(self.refstar.keys()):
            self.sc.starcirc[key].remove()
            del self.sc.starcirc[key]
            del self.refstar[key]
          self.sc.canvas.draw_idle()
        self.refstars = a.output 

    def button_click_mapp(self):
        self.datetime()
        if self.mapparam != []:
          a = MapDialog(magparams = self.mapparam)                
          a.exec_()

          if self.mapparam != a.output: 
            self.mapparam = a.output
            self.mapparam.append(str(self.elements[0].text()))
            self.MapPlot()

## *******************************************
############## Dialog classes ################
## *******************************************

class VisMeasure(QtWidgets.QDialog):
    def __init__(self, typ, parent = None):
        super(VisMeasure, self).__init__()
        self.element = {} 
        
        grid = QtWidgets.QGridLayout()
        names = [ 'Input Stars', 'Input Tail', 'Input Coma Again', 'Input Tail Again' ]
        if typ == 'comet':
          for i in range(3):
            self.element[i] = QtWidgets.QPushButton(names[i], self)
            grid.addWidget(self.element[i], *(i,0))
        elif typ == 'tail':
          names.remove('Input Coma Again')
          names.remove('Input Tail')
          for i in range(2):
            self.element[i] = QtWidgets.QPushButton(names[i], self)
            self.element[i].setFont(font)
            grid.addWidget(self.element[i], *(i,0))
        self.setLayout(grid)  
        self.setWindowTitle('Estimate choices')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

#plotting of comet map parameters
class MapDialog(QtWidgets.QDialog):
    def __init__(self, magparams, parent = None):
        super(MapDialog, self).__init__()
        grid = QtWidgets.QGridLayout()
        self.input, names, title, self.output = {}, [ 'Map center RA(deg): ', 'Map center DE(deg): ', 'Map size(deg): ', 'Limit mag: ', 'Source: ' ], 'Map parameters', magparams

        for i in range(5):
            label = QtWidgets.QLabel(names[i])
            label.setFont(font)
            grid.addWidget(label, *(i,0))

            if i == 4:
                self.input[i] = QtWidgets.QComboBox(self)
                self.input[i].setStyleSheet("selection-color: black")
                self.input[i].setFixedWidth(150)
                self.input[i].addItem('TYC-2')
                self.input[i].addItem('UCAC4')
                self.input[i].addItem('YBSC5')
                if magparams[4] == 'UCAC4':
                    self.input[i].setCurrentIndex(1)
                elif magparams[4] == 'YBSC5':
                    self.input[i].setCurrentIndex(2)
            else:
                self.input[i] = QtWidgets.QLineEdit(self)
                self.input[i].setText( magparams[i] )
            self.input[i].setFont(font)
            grid.addWidget(self.input[i], *(i,1))
       
        btn = QtWidgets.QPushButton('Ok', self)
        btn.setFont(font)
        btn.clicked.connect(self.button_click_save)
        grid.addWidget(btn, *(i+1,0))

        btncanc = QtWidgets.QPushButton('Cancel', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)
        grid.addWidget(btncanc, *(i+1,1))
        
        self.setLayout(grid)  
        self.setWindowTitle(title)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

    def button_click_save(self):
        self.output = [ str(self.input[0].text()), str(self.input[1].text()), str(self.input[2].text()), str(self.input[3].text()), str(self.input[4].currentText()).split(',')[0], self.output[5], self.output[6] ]
        self.close()

class ScrollVisual(QtWidgets.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtWidgets.QScrollArea.__init__(self, parent)

        self.w = Visual(self)
        self.setWidget(self.w)    
